import { getParsedType, hasKeys, INVALID, isOneOf, OK, ParsedType } from "@postgresql-typed/util";
import { PgTPBase } from "../../util/PgTPBase.js";
import { PgTPConstructorBase } from "../../util/PgTPConstructorBase.js";
import { throwPgTPError } from "../../util/throwPgTPError.js";
class LineConstructorClass extends PgTPConstructorBase {
    constructor() {
        super();
    }
    _parse(context) {
        const [argument, ...otherArguments] = context.data, allowedTypes = [ParsedType.number, ParsedType.string, ParsedType.object], parsedType = getParsedType(argument);
        if (parsedType !== ParsedType.number && context.data.length !== 1) {
            this.setIssueForContext(context, context.data.length > 1
                ? {
                    code: "too_big",
                    exact: true,
                    maximum: 1,
                    received: context.data.length,
                    type: "arguments",
                }
                : {
                    code: "too_small",
                    exact: true,
                    minimum: 1,
                    received: context.data.length,
                    type: "arguments",
                });
            return INVALID;
        }
        if (!isOneOf(allowedTypes, parsedType)) {
            this.setIssueForContext(context, {
                code: "invalid_type",
                expected: allowedTypes,
                received: parsedType,
            });
            return INVALID;
        }
        switch (parsedType) {
            case "string":
                return this._parseString(context, argument);
            case "number":
                return this._parseNumber(context, argument, otherArguments);
            default:
                return this._parseObject(context, argument);
        }
    }
    _parseString(context, argument) {
        // Remove all whitespace
        argument = argument.replaceAll(/\s/g, "");
        if (/^{(?:-?\d+(\.\d+)?|NaN),(?:-?\d+(\.\d+)?|NaN),(?:-?\d+(\.\d+)?|NaN)}$/.test(argument)) {
            const [a, b, c] = argument
                .slice(1, -1)
                .split(",")
                .map(l => Number.parseFloat(l));
            if (a === 0 && b === 0) {
                this.setIssueForContext(context, {
                    code: "too_small",
                    inclusive: true,
                    minimum: 1,
                    received: 0,
                    type: "number",
                });
                return INVALID;
            }
            return OK(new LineClass(a, b, c));
        }
        this.setIssueForContext(context, {
            code: "invalid_string",
            expected: "LIKE {a,b,c}",
            received: argument,
        });
        return INVALID;
    }
    _parseNumber(context, argument, otherArguments) {
        const totalLength = otherArguments.length + 1;
        if (totalLength !== 3) {
            this.setIssueForContext(context, totalLength > 3
                ? {
                    code: "too_big",
                    exact: true,
                    maximum: 3,
                    received: totalLength,
                    type: "arguments",
                }
                : {
                    code: "too_small",
                    exact: true,
                    minimum: 3,
                    received: totalLength,
                    type: "arguments",
                });
            return INVALID;
        }
        for (const otherArgument of otherArguments) {
            const allowedTypes = [ParsedType.number], parsedType = getParsedType(otherArgument);
            if (!isOneOf(allowedTypes, parsedType)) {
                this.setIssueForContext(context, {
                    code: "invalid_type",
                    expected: allowedTypes,
                    received: parsedType,
                });
                return INVALID;
            }
        }
        const [a, b, c] = [argument, ...otherArguments];
        return OK(new LineClass(a, b, c));
    }
    _parseObject(context, argument) {
        if (this.isLine(argument))
            return OK(new LineClass(argument.a, argument.b, argument.c));
        const parsedObject = hasKeys(argument, [
            ["a", "number"],
            ["b", "number"],
            ["c", "number"],
        ]);
        if (parsedObject.success)
            return OK(new LineClass(parsedObject.obj.a, parsedObject.obj.b, parsedObject.obj.c));
        switch (true) {
            case parsedObject.otherKeys.length > 0:
                this.setIssueForContext(context, {
                    code: "unrecognized_keys",
                    keys: parsedObject.otherKeys,
                });
                break;
            case parsedObject.missingKeys.length > 0:
                this.setIssueForContext(context, {
                    code: "missing_keys",
                    keys: parsedObject.missingKeys,
                });
                break;
            case parsedObject.invalidKeys.length > 0:
                this.setIssueForContext(context, {
                    code: "invalid_key_type",
                    ...parsedObject.invalidKeys[0],
                });
                break;
        }
        return INVALID;
    }
    isLine(object) {
        return object instanceof LineClass;
    }
}
const Line = new LineConstructorClass();
class LineClass extends PgTPBase {
    constructor(_a, _b, _c) {
        super();
        this._a = _a;
        this._b = _b;
        this._c = _c;
    }
    _equals(context) {
        //@ts-expect-error - _equals receives the same context as _parse
        const parsed = Line.safeFrom(...context.data);
        if (parsed.success) {
            return OK({
                data: parsed.data,
                equals: parsed.data.toString() === this.toString(),
            });
        }
        this.setIssueForContext(context, parsed.error.issue);
        return INVALID;
    }
    toString() {
        return `{${this._a},${this._b},${this._c}}`;
    }
    toJSON() {
        return {
            a: this._a,
            b: this._b,
            c: this._c,
        };
    }
    get a() {
        return this._a;
    }
    set a(a) {
        const parsedType = getParsedType(a);
        if (parsedType !== ParsedType.number) {
            throwPgTPError({
                code: "invalid_type",
                expected: [ParsedType.number],
                received: parsedType,
            });
        }
        this._a = a;
    }
    get b() {
        return this._b;
    }
    set b(b) {
        const parsedType = getParsedType(b);
        if (parsedType !== ParsedType.number) {
            throwPgTPError({
                code: "invalid_type",
                expected: [ParsedType.number],
                received: parsedType,
            });
        }
        this._b = b;
    }
    get c() {
        return this._c;
    }
    set c(c) {
        const parsedType = getParsedType(c);
        if (parsedType !== ParsedType.number) {
            throwPgTPError({
                code: "invalid_type",
                expected: [ParsedType.number],
                received: parsedType,
            });
        }
        this._c = c;
    }
    get value() {
        return this.toString();
    }
    set value(line) {
        const parsed = Line.safeFrom(line);
        if (parsed.success) {
            this._a = parsed.data.a;
            this._b = parsed.data.b;
            this._c = parsed.data.c;
        }
        else
            throw parsed.error;
    }
    get postgres() {
        return this.toString();
    }
    set postgres(line) {
        const parsed = Line.safeFrom(line);
        if (parsed.success) {
            this._a = parsed.data.a;
            this._b = parsed.data.b;
            this._c = parsed.data.c;
        }
        else
            throw parsed.error;
    }
}
export { Line };
//# sourceMappingURL=Line.js.map